# Support Minio: Override function of pandas 0.23.4
from pandas.io import s3
import os


def get_filepath_or_buffer(filepath_or_buffer, encoding=None,
                           compression=None, mode=None):
    from pandas import compat
    from botocore.exceptions import NoCredentialsError
    from s3fs import S3FileSystem

    if mode is None:
        mode = 'rb'
    ENDPOINT = os.environ['AWS_ENDPOINT']
    if not ENDPOINT.startswith('http'):
        ENDPOINT = 'https://' + ENDPOINT
    MINIO_ACCESS_KEY = os.environ['AWS_ACCESS_KEY_ID']
    MINIO_SECRET_KEY = os.environ['AWS_SECRET_ACCESS_KEY']
    fs = S3FileSystem(anon=False,
                      client_kwargs={'endpoint_url': ENDPOINT},
                      key=MINIO_ACCESS_KEY,
                      secret=MINIO_SECRET_KEY)
    try:
        filepath_or_buffer = fs.open(
            s3._strip_schema(filepath_or_buffer), mode)
    except (compat.FileNotFoundError, NoCredentialsError):
        # boto3 has troubles when trying to access a public file
        # when credentialed...
        # An OSError is raised if you have credentials, but they
        # aren't valid for that bucket.
        # A NoCredentialsError is raised if you don't have creds
        # for that bucket.
        fs = S3FileSystem(anon=True)
        filepath_or_buffer = fs.open(
            s3._strip_schema(filepath_or_buffer), mode)
    return filepath_or_buffer, None, compression, True


s3.get_filepath_or_buffer = get_filepath_or_buffer
