from dataclasses import dataclass
from typing import List
from enum import Enum
from aiaengine.common import Resource
from aiaengine.api import artifact as api_artifact
from google.protobuf import json_format

class ArtifactType(str, Enum):
    DataAnalysisResult = 'data_analysis_result'
    ClusteringResult = 'clustering_result'
    ClusterInsights = 'cluster_insights'

@dataclass
class Metadata:
    name: str
    labels: dict = None
    annotations: dict = None

class Artifact(Resource):
    """The object model represents an artifact in the Engine
    """

    @property
    def type(self) -> str:
        """Type of the artifact

        Returns:
            Type of the artifact
        """
        return self._data.type

    @property
    def metadata(self) -> Metadata:
        """Metadata of the artifact

        Returns:
            Metadata of the artifact
        """
        return Metadata(
            name=self._data.metadata.name,
            labels=self._data.metadata.labels,
            annotations=self._data.metadata.annotations
        )

    @property
    def data(self) -> dict:
        """Data of the artifact

        Returns:
            Data of the artifact
        """
        artifact_data = json_format.MessageToDict(self._data.data)
        return artifact_data

    def _get_data(self) -> any:
        return self.client.artifacts.GetArtifact(api_artifact.GetArtifactRequest(id=self.id))

    def __repr__(self) -> str:
        return f"Artifact(id='{self.id}', type='{self.type}')"