from enum import Enum
from aiaengine.api import Client

DEFAULT_TIMEOUT = 60 * 10
STATUS_SUCCESS = "success"
STATUS_FAILED = "failed"
STATUS_ACTIVE = "active"

PIPELINE_RUN_STATUS_PROCESSING = 0
PIPELINE_RUN_STATUS_SUCCESS = 1
PIPELINE_RUN_STATUS_FAILED = 2

LATEST_VERSION = -1

RECOMMENDED_FEATURE_SET = "Recommended features"

class Classifiers(str, Enum):
    ADABOOST = "adaboost_clf"
    XGBOOST = "xgboosting_clf"
    XGBOOST_GPU = "xgboosting_gpu_clf"
    RANDOM_FOREST = "random_forest_clf"

class Regressors(str, Enum):
    ADABOOST = "adaboost_regression"
    XGBOOST = "xgboosting_regression"
    XGBOOST_GPU = "xgboosting_gpu_regression"
    RANDOM_FOREST = "random_forest_regression"

class Clusterers(str, Enum):
    K_MEANS = "kmeans_clustering_spark"
    GMM = "gmm_clustering_spark"
    UMAP_HDBSCAN = "umap_hdbscan_clustering"

class Resource:
    def __init__(self, id: str, data: any = None, client: Client = None) -> None:
        self.id = id

        if client is None:
            client = get_default_client()
        self.client = client

        if data is None:
            data = self._get_data()
        self._data = data

    def _get_data(self) -> any:
        return None

_default_client = None

def get_default_client():
    global _default_client
    if _default_client is None:
        _default_client = Client()
    return _default_client