import os
from dataclasses import dataclass
from typing import List
from aiaengine.core import Schema
from aiaengine.io_utils import save_json, load_json

PREPROCESSING_PIPELINE_FORMAT_SCIKIT_LEARN = 'scikit-learn'
PREPROCESSING_PIPELINE_FORMAT_SPARKML = 'SparkML'
@dataclass(frozen=True)
class PreprocessingPipeline:
    """The executed (fitted) instance of the PreprocessingPipelineSpec on a specific input dataset
    """
    url: str
    format: str
    spec: dict = None
    input_schema: Schema = None
    transformed_feature_names: List[str] = None
    target_encodings: any = None

    @staticmethod
    def _get_metadata_path(url):
        return os.path.join(url, 'metadata.json')

    def save(self):
        """Save preprocessing pipeline"""
        metadata = {
            'input_schema': self.input_schema.to_dict(),
            'spec': self.spec.spec,
            'transformed_feature_names': self.transformed_feature_names,
            'format': self.format,
            'target_encodings': self.target_encodings
        }
        save_json(metadata, PreprocessingPipeline._get_metadata_path(self.url))

    def load(url):
        """Load a PreprocessingPipeline form the URL
        Args:
            url ([type]): [description]

        Returns:
            [PreprocessingPipeline]: [description]
        """
        metadata = load_json(PreprocessingPipeline._get_metadata_path(url))
        input_schema = Schema.from_dict(metadata['input_schema'])
        transformed_feature_names = metadata['transformed_feature_names']
        format = metadata['format']
        target_encodings = metadata['target_encodings']
        return PreprocessingPipeline(url=url,
                                     format=format,
                                     spec=metadata['spec'],
                                     input_schema=input_schema,
                                     transformed_feature_names=transformed_feature_names,
                                     target_encodings=target_encodings)
