from typing import List, Dict
from aiaengine.core import Dataset

PRIMARY_DATASET_ALIAS = 'pix.wrangling.main.0'

def is_primary_dataset(alias):
    return alias in ('', PRIMARY_DATASET_ALIAS)

class Recipe:
    """An object model represents a Recipe in the platform
    """
    def __init__(self,
            actions: List[dict], 
            input_datasets: Dict[str, Dataset],
            id: str=None,
            name: str=None,
            last_iteration: int=1,
            project_id: str=None):
        """
        Args:
            actions: List of commited actions
            input_datasets: Dictionary with key is alias of dataset
        """
        self.id = id
        self.name = name
        self.project_id = project_id
        self.actions = actions
        self.input_datasets = input_datasets
        self.last_iteration = last_iteration
        self.output_dataset_id = None

    def get_primary_dataset(self):
        """
            Get primary dataset from input datasets
        """
        return self.input_datasets[PRIMARY_DATASET_ALIAS]

    def get_secondary_datasets(self):
        """
            Get secondary dataset from input datasets
        """
        return {alias:ds for alias,ds in self.input_datasets.items() if alias != PRIMARY_DATASET_ALIAS}