import abc

from google.protobuf import json_format
from aiaengine import api
from aiaengine.api import artifact as artifact_api
from aiaengine.api.artifact_pb2 import Metadata
from aiaengine.core import Dataset, Artifact, Metadata
from aiaengine.pipeline import format_object_input, register_converter
from aiaengine.pipeline.converters.platform import ARTIFACT_OBJECT_TYPE
from aiaengine.pipeline.converters.base import BasePlatformObjectConverter


class ArtifactLoader(abc.ABC):
    @abc.abstractmethod
    def load(self, id: str) -> Artifact:
        raise Exception("Not implemented")


class PlatformArtifactLoader(ArtifactLoader):
    def __init__(self, client: api.Client):
        self.client = client

    def load(self, id: str, dataset_version: int = -1) -> Dataset:
        artifact_info = self.client.artifacts.GetArtifact(artifact_api.GetArtifactRequest(id=id))

        return Artifact(
            id=id,
            type=artifact_info.type,
            metadata=Metadata(
                name=artifact_info.metadata.name,
                labels=artifact_info.metadata.labels,
                annotations=artifact_info.metadata.annotations,
            ),
            data=json_format.MessageToDict(artifact_info.data)
        )


class ArtifactConverter(BasePlatformObjectConverter):
    def __init__(self, loader: ArtifactLoader):
        super().__init__(object_type=ARTIFACT_OBJECT_TYPE)
        self.loader = loader

    def convert_from(self, value: str):
        json_data = self.parse_json(value)
        return self.loader.load(json_data["id"])

    def convert_to(self, value: Artifact):
        return format_object_input(value.id)


def register_artifact_converter(loader: ArtifactLoader):
    register_converter(ARTIFACT_OBJECT_TYPE, ArtifactConverter(loader=loader))
