import json
from aiaengine.pipeline import Converter, format_object_input

class IntegerConverter(Converter):
    def convert_from(self, value: str):
        return int(value)

    def convert_to(self, value: int):
        return str(value)

class FloatConverter:
    def convert_from(self, value: str):
        return float(value)

    def convert_to(self, value: float):
        return str(value)

class StringConverter(Converter):
    def convert_from(self, value: str):
        return value

    def convert_to(self, value: any):
        return value

class BooleanConverter(Converter):
    def convert_from(self, value: str):
        return value == 'True' or value == 'true' or value == '1'
    
    def convert_to(self, value: bool):
        return 'true' if value else 'false'

class JSONConverter(Converter):
    def convert_from(self, value: str):
        return json.loads(value)

    def convert_to(self, value: dict):
        return json.dumps(value)

class BasePlatformObjectConverter(Converter):
    def __init__(self, object_type) -> None:
        self.object_type = object_type

    def parse_json(self, value: str) -> dict:
        json_data = json.loads(value)
        if json_data['type'] != self.object_type:
            raise Exception(f"invalid type: {json_data['type']}, expect {self.object_type}")
        if json_data['id'] is None:
            raise Exception('id is required')
        return json_data

    def convert_to(self, value: any):
        return format_object_input(self.object_type, value.id)