from typing import Dict, Union
import os

is_gpu_env = os.environ.get('GPU_ENV')
if is_gpu_env:
    import cudf

from aiaengine.core import Dataset, Schema
from aiaengine.io_utils import get_storage_options
from aiaengine.pipeline.converters.dataset import DatasetConverter, DatasetLoader


class CumlDataset(Dataset):
    def __init__(self, dataset: Dataset, df: cudf = None):
        super().__init__(schema=dataset.schema,
                         data_urls=dataset.data_urls,
                         id=dataset.id,
                         version=dataset.version)
        self._df = df

    @staticmethod
    def from_dataframe(df: cudf, schema: Union[Dict, Schema]):
        if not isinstance(schema, Schema):
            schema = Schema.from_dict(schema)
        return CumlDataset(Dataset(schema=schema, data_urls=None), df=df)

    def get_df(self):
        if self._df is None:
            list_dfs = [cudf.read_parquet(file_path,
                                        storage_options=get_storage_options(file_path))
                        for file_path in self.data_urls]
            self._df = cudf.concat(list_dfs, ignore_index=True)
        return self._df


class CumlDatasetConverter(DatasetConverter):
    def __init__(self, loader: DatasetLoader):
        super().__init__(loader=loader)

    def convert_from(self, val: str) -> CumlDataset:
        dataset = super().convert_from(val)
        return CumlDataset(dataset=dataset)

    def convert_to(self, val: CumlDataset) -> CumlDataset:
        return super().convert_to(val)
