from typing import Dict, Union

import dask.dataframe as dd
from aiaengine.core import Dataset, Schema
from aiaengine.pipeline import register_converter
from aiaengine.pipeline.converters.dataset import DatasetConverter, DatasetLoader
from aiaengine.io_utils import get_storage_options


class DaskDataset(Dataset):
    def __init__(self, dataset: Dataset, df: dd.DataFrame=None):
        super().__init__(schema=dataset.schema,
                         data_urls=dataset.data_urls,
                         id=dataset.id,
                         version=dataset.version)
        self._df = df

    @staticmethod
    def from_dataframe(df: dd.DataFrame, schema: Union[Dict, Schema]):
        if not isinstance(schema, Schema):
            schema = Schema.from_dict(schema)
        return DaskDataset(Dataset(schema=schema, data_urls=None), df=df)

    def get_df(self):
        if self._df is None:
            list_dfs = [dd.read_parquet(file_path,
                                        storage_options=get_storage_options(file_path))
                        for file_path in self.data_urls]
            self._df = dd.concat(list_dfs, ignore_index=True)
        return self._df


class DaskDatasetConverter(DatasetConverter):
    def __init__(self, loader: DatasetLoader):
        super().__init__(loader=loader)

    def convert_from(self, val: str) -> DaskDataset:
        dataset = super().convert_from(val)
        return DaskDataset(dataset=dataset)

    def convert_to(self, val: DaskDataset) -> DaskDataset:
        return super().convert_to(val)

def register_dask_dataset_converter(converter: DaskDatasetConverter):
    register_converter('DaskDataset', converter)
