from typing import Dict, Union
import os

is_gpu_env = os.environ.get('GPU_ENV')
if is_gpu_env:
    import dask_cudf

from aiaengine.core import Dataset, Schema
from aiaengine.pipeline import register_converter
from aiaengine.io_utils import get_storage_options
from aiaengine.pipeline.converters.dataset import DatasetConverter, DatasetLoader

class DaskCudfDataset(Dataset):
    def __init__(self, dataset: Dataset, df: dask_cudf.DataFrame = None):
        super().__init__(schema=dataset.schema,
                         data_urls=dataset.data_urls,
                         id=dataset.id,
                         version=dataset.version)
        self._df = df

    @staticmethod
    def from_dataframe(df: dask_cudf.DataFrame, schema: Union[Dict, Schema]):
        if not isinstance(schema, Schema):
            schema = Schema.from_dict(schema)
        return DaskCudfDataset(Dataset(schema=schema, data_urls=None), df=df)

    def get_df(self):
        if self._df is None:
            list_dfs = [dask_cudf.read_parquet(file_path,
                                        storage_options=get_storage_options(file_path))
                        for file_path in self.data_urls]
            self._df = dask_cudf.concat(list_dfs, ignore_index=True)
        return self._df


class DaskCudfDatasetConverter(DatasetConverter):
    def __init__(self, loader: DatasetLoader):
        super().__init__(loader=loader)

    def convert_from(self, val: str) -> DaskCudfDataset:
        dataset = super().convert_from(val)
        return DaskCudfDataset(dataset=dataset)

    def convert_to(self, val: DaskCudfDataset) -> DaskCudfDataset:
        return super().convert_to(val)

def register_dask_cudf_dataset_converter(converter: DaskCudfDatasetConverter):
    register_converter('DaskCudfDataset', converter)