from typing import Dict, Union

import pandas as pd
from aiaengine.core import Dataset, Schema
from aiaengine.pipeline.converters.dataset import DatasetConverter, DatasetLoader
from aiaengine.io_utils import get_storage_options


class PandasDataset(Dataset):
    def __init__(self, dataset: Dataset, df: pd.DataFrame=None):
        super().__init__(schema=dataset.schema,
                         data_urls=dataset.data_urls,
                         id=dataset.id,
                         version=dataset.version)
        self._df = df

    @staticmethod
    def from_dataframe(df: pd.DataFrame, schema: Union[Dict, Schema]):
        if not isinstance(schema, Schema):
            schema = Schema.from_dict(schema)
        return PandasDataset(Dataset(schema=schema, data_urls=None), df=df)

    def get_df(self):
        if self._df is None:
            list_dfs = [pd.read_parquet(file_path,
                                        storage_options=get_storage_options(file_path))
                        for file_path in self.data_urls]
            self._df = pd.concat(list_dfs, ignore_index=True)
        return self._df


class PandasDatasetConverter(DatasetConverter):
    def __init__(self, loader: DatasetLoader):
        super().__init__(loader=loader)

    def convert_from(self, val: str) -> PandasDataset:
        dataset = super().convert_from(val)
        return PandasDataset(dataset=dataset)

    def convert_to(self, val: PandasDataset) -> PandasDataset:
        return super().convert_to(val)
