import abc
from aiaengine import api
from aiaengine.api import preprocessing_pipeline
from aiaengine.core import PreprocessingPipelineSpec
from aiaengine.pipeline import register_converter
from aiaengine.pipeline.converters.feature_set import FeatureSetLoader
from aiaengine.pipeline.converters.base import BasePlatformObjectConverter
from google.protobuf import json_format
from aiaengine.pipeline.converters.platform import PREPROCESSING_PIPELINE_SPEC_OBJECT_TYPE

class PreprocessingPipelineSpecLoader:
    @abc.abstractmethod
    def load(self, id: str) -> PreprocessingPipelineSpec:
        pass

class PlatformPreprocessingPipelineSpecLoader(PreprocessingPipelineSpecLoader):
    def __init__(self,
        client: api.Client,
        feature_set_loader: FeatureSetLoader):
        self.client = client
        self.feature_set_loader = feature_set_loader

    def load(self, id: str) -> PreprocessingPipelineSpec:
        data = self.client.preprocessing_pipelines.GetPreprocessingPipelineSpec(preprocessing_pipeline.GetPreprocessingPipelineSpecRequest(id=id))
        feature_set = self.feature_set_loader.load(id=data.feature_set.id)

        return PreprocessingPipelineSpec(
            id=id,
            name=data.name,
            description=data.description,
            spec=json_format.MessageToDict(data.spec),
            feature_set=feature_set
        )

class PreprocessingPipelineSpecConverter(BasePlatformObjectConverter):
    """Convert JSON representation of feature set <-> PreprocessingPipelineSpec object"""
    def __init__(self, loader: PreprocessingPipelineSpecLoader):
        super().__init__(object_type=PREPROCESSING_PIPELINE_SPEC_OBJECT_TYPE)
        self.loader = loader

    def convert_from(self, value: str):
        json_data = self.parse_json(value)
        return self.loader.load(json_data['id'])

def register_preprocessing_pipeline_spec_converter(loader: PreprocessingPipelineSpecLoader):
    register_converter(PREPROCESSING_PIPELINE_SPEC_OBJECT_TYPE, PreprocessingPipelineSpecConverter(loader=loader))