"""
Mappings of semantic types to dtypes
and between dtypes of spark
"""

from pyspark.sql.types import \
    TimestampType, StringType, \
    BooleanType, NumericType, DataType, TimestampNTZType

EXISTING_DATA_TYPES = [
    'DateTime',
    'Numeric',
    'Text',
    'Boolean'
]

DATA_TYPE_TO_SPARK_TYPE = {
    'Numeric': NumericType,
    'DateTime': TimestampType,
    'Text': StringType,
    'Boolean': BooleanType,
}

assert sorted(EXISTING_DATA_TYPES) == sorted(DATA_TYPE_TO_SPARK_TYPE), (
    'Mismatch between EXISTING_DATA_TYPES and DATA_TYPE_TO_SPARK_TYPE mapping.'
)


def spark_type_agrees_with_data_type(dtype: DataType, data_type: str):
    """
    Check if the Spark dtype is aligned with data type
    """
    assert data_type in EXISTING_DATA_TYPES, (
        f'Unrecognised data type {data_type}'
    )

    # handle specific for TimestampType
    return (data_type == "DateTime" and isinstance(dtype, (TimestampNTZType, TimestampType))) \
        or isinstance(dtype, DATA_TYPE_TO_SPARK_TYPE[data_type])
