import os
from aiaengine.api import Client
from aiaengine.pipeline import ServiceProvider, register_service_provider
from aiaengine.pipeline.providers.artifact_writer import ArtifactWriterProvider, ArtifactWriter

from aiaengine.pipeline.providers.preprocessing_pipeline import (
    PreprocessingPipelineSpecWriter, PlatformPreprocessingPipelineSpecWriter, PreprocessingPipelineSpecWriterProvider,
    PreprocessingPipelineWriter, PlatformPreprocessingPipelineWriter, PreprocessingPipelineWriterProvider
)

from aiaengine.pipeline.providers.model import (
    ModelWriter, PlatformModelWriter, ModelWriterProvider,
    PlatformPredictionResultWriter, PredictionResultWriterProvider, TrainingResultWriter, PlatformTrainingResultWriter, TrainingResultWriterProvider,
    EvaluationResultWriter, PlatformEvaluationResultWriter, EvaluationResultWriterProvider,
)

from aiaengine.pipeline.providers.cluster_factory import ClusterFactory, ClusterFactoryServiceProvider
from aiaengine.pipeline.providers.dataset_reader import (
    DatasetReaderFactory,
    DatasetReaderFactoryServiceProvider,
    DATAFRAME_FORMAT_PANDAS,
    DATAFRAME_FORMAT_DASK,
    DATAFRAME_FORMAT_SPARK,
    DATAFRAME_FORMAT_CUDF,
    DATAFRAME_FORMAT_DASK_CUDF
)

from aiaengine.pipeline.providers.feature_set import FeatureSetWriter, FeatureSetWriterProvider
from aiaengine.pipeline.providers.progress_reporter import  ProgressReporter, ProgressReporterProvider
from aiaengine.pipeline.providers.app import AppWriter, AppWriterProvider
from aiaengine.pipeline.providers.recipe import RecipeWriter, RecipeWriterProvider

class ApiClientServiceProvider(ServiceProvider):
    def __init__(self, api_client: Client):
        self._client = api_client

    def get_instance(self) -> Client:
        return self._client


def register_service_providers(client: Client, clusters={}, dataset_readers={}):
    register_service_provider('Client', ApiClientServiceProvider(client))

    register_service_provider('FeatureSetWriter', FeatureSetWriterProvider(client))

    register_service_provider('PreprocessingPipelineSpecWriter',
                              PreprocessingPipelineSpecWriterProvider(writer=PlatformPreprocessingPipelineSpecWriter(client=client)))

    register_service_provider('PreprocessingPipelineWriter',
                              PreprocessingPipelineWriterProvider(writer=PlatformPreprocessingPipelineWriter(client=client)))

    register_service_provider('ModelWriter',
                              ModelWriterProvider(writer=PlatformModelWriter(client=client)))
    register_service_provider('TrainingResultWriter',
                              TrainingResultWriterProvider(writer=PlatformTrainingResultWriter(client=client)))
    register_service_provider('EvaluationResultWriter',
                              EvaluationResultWriterProvider(writer=PlatformEvaluationResultWriter(client=client)))
    register_service_provider('PredictionResultWriter',
                              PredictionResultWriterProvider(writer=PlatformPredictionResultWriter(client=client)))

    register_service_provider('ClusterFactory', ClusterFactoryServiceProvider(clusters=clusters))
    register_service_provider('DatasetReaderFactory', DatasetReaderFactoryServiceProvider(readers=dataset_readers))

    register_service_provider('AppWriter', AppWriterProvider(client=client))
    register_service_provider('RecipeWriter', RecipeWriterProvider(client=client))

    register_service_provider('ArtifactWriter', ArtifactWriterProvider(client=client))

    register_service_provider('ProgressReporter', ProgressReporterProvider(client=client, pipelinerun_id=os.environ.get('PIPELINERUN_ID')))
