from __future__ import annotations
import dataclasses
from google.protobuf import struct_pb2, json_format
from aiaengine.api import Client, app_pb2 as app_api
from aiaengine.core import App
from aiaengine.pipeline.providers import ServiceProvider

class AppWriter:
    def write(self, app: App) -> App:
        return app

class PlatformAppWriter(AppWriter):
    def __init__(self, client: Client):
        self.client = client

    def write(self, app: App):
        if app.id is not None:
            api_problem_type_config = None
            if app.problem_type_config:
                api_problem_type_config = struct_pb2.Struct()
                json_format.ParseDict(
                    dataclasses.asdict(app.problem_type_config), api_problem_type_config
                )
            dataset_id = app.dataset.id if app.dataset else None
            _ = self.client.apps.UpdateApp(
                app_api.UpdateAppRequest(
                    id=app.id,
                    name=app.name,
                    description=app.description,
                    dataset_id=dataset_id,
                    annotations=app.annotations,
                    problem_type=app.problem_type,
                    problem_type_config=api_problem_type_config
                )
            )
            return app

class AppWriterProvider(ServiceProvider):
    def __init__(self, client: Client):
        self.client = client

    def get_instance(self) -> PlatformAppWriter:
        return PlatformAppWriter(client=self.client)