from __future__ import annotations
import dataclasses
from aiaengine.api import Client, artifact_pb2 as artifact_api
from aiaengine.core import Artifact
from aiaengine.pipeline.providers import ServiceProvider
from google.protobuf import json_format, struct_pb2

class ArtifactWriter:
    def write(self, artifact: Artifact) -> Artifact:
        return artifact

class PlatformArtifactWriter(ArtifactWriter):
    def __init__(self, client: Client):
        self.client = client

    def write(self, artifact: Artifact):
        data = struct_pb2.Struct()
        json_format.ParseDict(artifact.data, data)
        metadata = artifact_api.Metadata(
            name=artifact.metadata.name,
            labels=artifact.metadata.labels,
            annotations=artifact.metadata.annotations
        )
        if artifact.id is None:
            response = self.client.artifacts.CreateArtifact(
                artifact_api.CreateArtifactRequest(
                    type=artifact.type,
                    metadata=metadata,
                    data=data,
                )
            )
            return dataclasses.replace(artifact, id=response.id)
        else:
            response = self.client.artifacts.UpdateArtifact(
                artifact_api.UpdateArtifactRequest(
                    id=artifact.id,
                    metadata=metadata,
                    data=data,
                )
            )
            return artifact

class ArtifactWriterProvider(ServiceProvider):
    def __init__(self, client: Client):
        self.client = client

    def get_instance(self) -> PlatformArtifactWriter:
        return PlatformArtifactWriter(client=self.client)