from abc import abstractmethod
from aiaengine.core import Dataset
from aiaengine.pipeline import ServiceProvider


DATAFRAME_FORMAT_PANDAS = 'Pandas'
DATAFRAME_FORMAT_DASK = 'Dask'
DATAFRAME_FORMAT_SPARK = 'Spark'
DATAFRAME_FORMAT_CUDF = 'cuDF'
DATAFRAME_FORMAT_DASK_CUDF = 'Dask_cuDF'

class DatasetReader:
    @abstractmethod
    def read(self, dataset: Dataset) -> any:
        """Read dataset into the memory in expected dataframe format

        Args:
            dataset (Dataset): [description]

        Returns:
            any: [description]
        """

class DatasetReaderFactory:
    def __init__(self, readers={}):
        self.readers = readers

    def register_reader(self, dataframe_format: str, reader: DatasetReader):
        self.readers[dataframe_format] = reader

    def get_reader(self, dataframe_format: str) -> DatasetReader:
        if dataframe_format in self.readers:
            return self.readers[dataframe_format]
        else:
            raise Exception(f'No registered reader of dataframe format {dataframe_format}')

class DatasetReaderFactoryServiceProvider(ServiceProvider):
    def __init__(self, readers={}):
        self.readers = readers

    def get_instance(self) -> DatasetReaderFactory:
        return DatasetReaderFactory(readers=self.readers)