import logging
from aiaengine.api import Client
from aiaengine.util import report_progress_async, report_progress
from aiaengine.pipeline.providers import ServiceProvider

class ProgressReporter:
    def report(self, task_name: str, progress: float, total: float, is_async=True):
        logging.info(f'Report progress for task={task_name}, progress={progress}, total={total}')

class PlatformProgressReporter(ProgressReporter):
    def __init__(self, client: Client, pipelinerun_id: str):
        self.client = client
        self.pipelinerun_id = pipelinerun_id

    def report(self, task_name: str, progress: float, total: float, is_async=True):
        if total == 0:
            logging.warning('Progress `total` cannot be zero.')
            return
        report_fn = report_progress_async if is_async else report_progress
        report_fn(api_client=self.client, task_name=task_name,
                progress=(progress * 100/ total),
                pipelinerun_id=self.pipelinerun_id)


class ProgressReporterProvider(ServiceProvider):
    def __init__(self, client: Client, pipelinerun_id: str):
        self.client = client
        self.pipelinerun_id = pipelinerun_id

    def get_instance(self) -> ProgressReporter:
        return PlatformProgressReporter(self.client, self.pipelinerun_id)

    