from abc import ABC
import json

from .column_selectors import name_equals

class Input(ABC):
    def __init__(self, select, key='operand', dataset='pix.wrangling.main.0') -> None:
        self.key = key
        self.dataset = dataset
        if isinstance(select, str):
            self.select = name_equals(select)
        else:
            self.select = select
        pass

class Output(ABC):
    def __init__(self, name: str, dataset='pix.wrangling.main.0') -> None:
        self.name = name
        self.dataset = dataset
        pass

class Action(ABC):
    def __init__(self, name, input, output, parameters) -> None:
        self.name = name
        self.input = input
        self.output = output
        self.parameters = parameters
        pass

    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__, 
            sort_keys=True, indent=4)

class ActionBuilder(ABC):
    def __init__(self, name) -> None:
        self.name = name
        self.inputs = []
        self.outputs = []
        self.parameters = {}

    def add_input_output(self, key, inputs_outputs):
        if inputs_outputs is None:
            return

        if not isinstance(inputs_outputs, list):
            inputs_outputs = [inputs_outputs]
        for input_output_pair in inputs_outputs:
            if isinstance(input_output_pair, tuple):
                input = input_output_pair[0]
                output = input_output_pair[1]
            else:
                input = input_output_pair
                output = None
            if not isinstance(input, list):
                input = [input]
            if not isinstance(output, list):
                output = [output]
            for ip in input:
                if ip is not None:
                    self.inputs.append(Input(select=ip, key=key))
            for op in output:
                if op is not None:
                    self.outputs.append(Output(name=op))

    def add_input(self, selector):
        self.inputs.append(Input(selector))
        return self

    def add_output(self, name):
        self.outputs.append(Output(name))
        return self

    def set_parameter(self, name: str, value):
        self.parameters[name] = value
        return self

    def build(self):
        return Action(name=self.name, 
            input=self.inputs,
            output=self.outputs,
            parameters=self.parameters)