import json
from jinja2 import Template

def gen_actions(actions):
    template = Template("""# WARNING: THIS CODE IS AUTOMATICALLY GENERATED. DO NOT EDIT.
from .actions import ActionBuilder
{% for action in actions %}
def {{ action['name'] }}({% if action['input_output_editors'] | length > 0 %}{% for pair in action['input_output_editors'] %}{% if loop.index0 > 0 %}, {% endif %}{{ pair['key'] }}=None{% endfor %},{% endif %}{% for param in action['parameters'] %}{% if loop.index0 > 0 %}, {% endif %}{{ param['name'] }}=None{% endfor %}):
    \"""
    {{ action['description'] }}
    Input/Output:
        operand = ([input1, input2, input3], [output1, output2])
        where
            - input can be a single column name or a single column selector
            - output is a column name or column prefix/suffix
        Example:
        {%- for pair in action['input_output_editors'] %}
            {{ pair['key'] }} = ([{% if  pair['input'] is not none %}{% for input in pair['input'] %}{% if loop.index0 > 0 %}, {% endif %}'input_column_{{ loop.index }}'{% endfor %}]{% endif %}{% if pair['output'] is not none %}, [{% for output in pair['output'] %}{% if loop.index0 > 0 %}, {% endif %}'output_{{ loop.index }}'{% endfor %}{% endif %}{% if  pair['output'] is not none %}]{% endif %})
        {%- endfor %}
    
    {% if action['parameters'] | length > 0 %}Parameters:{% endif %}
    {% for param in action['parameters'] -%}
    {{ param['name'] }}: {{ param['description'] }}
    {% endfor -%}
    \"""
    builder = ActionBuilder('{{ action['name'] }}')
    {% for pair in action['input_output_editors'] -%}
    builder.add_input_output('{{ pair['key'] }}', {{ pair['key'] }})
    {% endfor -%}
    {% for param in action['parameters'] -%}
    if {{ param['name'] }} is not None:
        builder.set_parameter('{{ param['name'] }}', {{ param['name'] }})
    {% endfor -%}
    return builder.build()
{% endfor %}
""")
    return template.render(actions=actions)

def gen():
    with open('actions.json') as f:
        actions = json.load(f)
        with open('generated_actions.py', 'w') as out:
            out.write(gen_actions(actions))
            out.close()

if __name__ == "__main__":
    gen()